/*
 * FileReader.h
 *
 *  Created on: Nov 11, 2013
 *      Author: mswindell
 */


#ifndef FILEREADER_H_
#define FILEREADER_H_

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <stdio.h>
#include "cdf.h"
#include "Attribute.h"
#include "Time.h"

class FileReader {
protected:
    static const float missingValue = -256. * -256. * -256. * -256. * -256.;
    float lat;
    float lon;
    CDFid currentFileId;
    std::string currentFilename;
    std::string variable;
    std::string filename;
    std::vector<std::string> loadedVariables;
    std::vector<long> loadedVariablesById;
    std::map<std::string, std::vector<float>* > variableDataMap;
    std::map<std::string, long> variableIds;
    std::map<long, std::string> variableNames;
    
    std::map<long, Attribute> gAttributeById;
    std::map<const std::string, Attribute> gAttributes;


public:
    FileReader();
    FileReader(int lat, int lon, std::string variable, std::string filename);
    virtual ~FileReader();
    void close();
    bool doesVariableExist(const std::string& varible);
    CDFid getCurrentFileId();
    std::string getCurrentFilename();
    Time getCurrentTime();
    Attribute getGlobalAttribute(long id);
    Attribute getGlobalAttribute(const std::string& attribute);
    long getGlobalAttributeID(const std::string& attribute);
    //std::vector<std::string> getListOfRequiredVariablesForComponents(std::string variable);
    std::vector<std::string> getListOfRequiredVariables(std::string variable);
    std::vector<std::string> getLoadedVariables();
    float getMissingValue();
    long getVariableId(const std::string& variable);
    std::vector<float>* getVariableData(const std::string variable);
    long interpolate();
    long loadVariable(const std::string& variable);
    long open(const std::string& filename);
    void initializeVariableMaps();
    void setCurrentFileId(CDFid id);
    
    
    bool unloadVariable(const std::string& variable);
    //Interpolator* createInterpolator();

    static const long OK = 0L;
    static const long OPEN_ERROR = -1L;
    static const long FILE_DOES_NOT_EXIST = -2L;
    static const long VARIABLE_DOES_NOT_EXIST = -3L;
    static const long ATTRIBUTE_DOES_NOT_EXIST = -4L;
    static const long LOAD_FAILED = -5L;
    static const long UNABLE_TO_ALLOCATE_MEMORY = -6L;
    static const long VARIABLE_NOT_IN_MEMORY = -7L;
    static const long MODEL_NOT_SUPPORTED = -8L;
    static const long NOT_A_VALID_KAMELEON_FILE = -9L;



};


#endif /* FILEREADER_H_ */





