/*
 * Interpolator.cpp
 *
 *  Created on: Nov 20, 2013
 *      Author: mswindell
 */

#include "Interpolator.h"
#include "FileReader.h"
#include "Utils.h"
#include <vector>
#include <math.h>

Interpolator::Interpolator(FileReader* fileReader) {
    this->fileReader = fileReader;
    //modelReader->setMissingValue(this->modelReader->getMissingValue());

    //latitude
    th_string = "theta";

    //longitude
    phi_string = "phi";

}

Interpolator::Interpolator(){
    fileReader = NULL;
}

Interpolator::~Interpolator() {
    // TODO Auto-generated destructor stub
}

long Interpolator::open(std::string& filename) {
    long status;
    if (fileReader != NULL) {
        if (filename == fileReader->getCurrentFilename())
            return 0;
        else {
            this->fileReader->close();
        }
    }
    this->fileReader = new FileReader();
    status = this->fileReader->open(filename);
    return status;
}

void Interpolator::close() {
    //std::cout<<"Interpolator::close: Beginning of close function."<<std::endl;
    this->fileReader->close();
    FileReader* fr = this->fileReader;
    fileReader = NULL;
    std::vector<float>* temp = th_data;
    th_data = NULL;
    std::vector<float>* temp1 = phi_data;
    phi_data = NULL;
    phi_size = 0;
    theta_size = 0;
    delete fr;
    //std::cout<<"Interpolator::close: Returning from close function."<<std::endl;
}

void Interpolator::getPositionData(){
    th_string = "theta";
    phi_string = "phi";
    th_data = fileReader->getVariableData(th_string);
    phi_data = fileReader->getVariableData(phi_string);
    theta_size = th_data->size();
    phi_size = phi_data->size();
}

void Interpolator::loadVariable(std::string& variable){
    this->fileReader->loadVariable(variable);
}

float Interpolator::interpolate(const std::string variable, const float& r, const float& lat, const float& lon){
    return interpolate((char*)variable.c_str(), &lat, &lon);
}

float Interpolator::interpolate(const char* var, const float* lat, const float* lon) {
    //std::cout<<"\t"<<*lat;
    //std::cout<<"\t"<<*lon;
    //so... assume the longitude is in magnetic local time, since that's what they want!float lo_tude = fmod((lon * 15. + 180), 360);
    
    /*
    So for each point I will have a co-latitude and a longitude. The radius I can assume is some constant.
    Then, the value that I need will come from basically any another variable, for example e-flux.

    Point 0: (psi <= lon, theta <= lat)
    Point 1: (psi >= lon, theta <= lat)
    Point 2: (psi >= lon, theta >= lat)
    Point 3: (psi <= lon, theta >= lat)

           <--
        3--------2
        |        |^
        |        ||
        0--------1
           -->
     */
     
    //std::cout<<"Inside Interpolator::interpolate:"<<std::endl;
    std::string variable(var);
    //std::cout<<"variable: "<<variable<<std::endl;
    
    getPositionData();
    float longitude = fmod((*lon * 15. + 180.), 360.);
    long status;
    float epsilon = 0.02, new_lat = *lat;
    //If the absolute value of the latitude is within 0.02 of 90, round it up to 90.
    //if(*lat - epsilon < 90.0 && *lat+epsilon >= 90.0){
    //  new_lat = 90;
    //}  
   
    float spole = (*th_data)[(*th_data).size() - 1];
    float npole = (*th_data)[0];

    if(spole < 90.0 && spole + 0.02 >= 90.0){
   //     (*th_data)[(*th_data).size()-1] = 90.0;
    }
    
    if(npole > -90.0 && npole - 0.02 <= -90.0){
     //   (*th_data)[0] = -90.0;
    }


     int ilon = Utils<float>::binary_search(*phi_data, 0, (*phi_data).size() - 1, longitude);
    int ilat = Utils<float>::binary_search(*th_data, 0, (*th_data).size() - 1, *lat);

    //the longitude and latitude were not found!
    //if (ilon < 0 || ilat < 0) {
      //  return this->fileReader->getMissingValue();
    //}

    //if(ilon < 0){
      //  if(longitude < (*phi_data)[0]){
      //      ilon = 0;
      //      std::cout<<"longitude = "<<longitude<<std::endl;
      //  } else if(longitude > (*phi_data)[(*phi_data).size()-1]){
      //      ilon = (*phi_data).size() - 2;
      //      std::cout<<"longitude = "<<longitude<<std::endl;
      //      std::cout<<"(*phi_data)[(*phi_data).size()-1 = "<<(*phi_data)[(*phi_data).size()-1]<<std::endl;
      //  }
   // }
    
   if(ilat < 0){
     if(*lat < (*th_data)[0]){
       ilat = 0;
       //std::cout<<"latitude  = "<<*lat<<std::endl;
      } else if(*lat > (*th_data)[(*th_data).size()-1] && (*th_data)[(*th_data).size()-1] > 89.9){
         float lat1 = (*th_data)[(*th_data).size()-1];
         return interpolate(var, &lat1, lon);
         //ilat = (*th_data).size() - 1;
         //std::cout<<"latitude = "<<*lat<<std::endl;
         //std::cout<<"(*th_data)[(*th_data).size()-1] = "<<(*th_data)[(*th_data).size()-1]<<std::endl;
      }
   }

    float lon1 = (*phi_data)[ilon];
    float lon2 = (*phi_data)[ilon + 1];

    float lat1 = (*th_data)[ilat];
    float lat2 = (*th_data)[ilat + 1];

    float lonP = (longitude - lon1) / (lon2 - lon1); //the percentage along the ray between lon1 and lon2
    //float latP = (*lat - lat1) / (lat2 - lat1);
    float latP = (new_lat - lat1) / (lat2 - lat1);

    //grab a pointer to the variable data so we can index into it.
    std::vector<float> * vData = NULL;
    vData = fileReader->getVariableData(variable);
    
    float d0 = (*vData)[index_2D_to_1D(ilat, ilon)];
    float d1 = (*vData)[index_2D_to_1D(ilat, ilon + 1)];
    float d2 = (*vData)[index_2D_to_1D(ilat + 1, ilon + 1)];
    float d3 = (*vData)[index_2D_to_1D(ilat + 1, ilon)];


    float p01 = d0 * (1.0 - lonP) + d1 * (lonP);
    float p32 = d3 * (1.0 - lonP) + d2 * (lonP);
    return p01 * (1.0 - latP) + p32 * (latP);

}

int Interpolator::index_2D_to_1D(const int& ilat, const int& ilon) {
    int index = ilon * theta_size + ilat;
    return index;
}

/*
int Interpolator::binary_search(const std::vector<float>& vec, unsigned int start, unsigned int end, const float& key) {
        // Termination condition: start index greater than end index

        if (start > end) {
            return -1;
        }

        // Find the middle element of the vector and use that for splitting
        // the array into two pieces.
        unsigned int middle = (start + ((end - start) / 2));

        //outside valid range
        if (middle > vec.size())
            return -1;

        if (key == vec[middle]) {
            return middle;
        } else if (key < vec[middle + 1] && key >= vec[middle]) {
            return middle;
        } else if (key < vec[middle]) {
            return binary_search(vec, start, middle - 1, key);
        }
        return binary_search(vec, middle + 1, end, key);
    }
 */

