#include "Interpolator_c.h"
#include "Interpolator.h"
#include <map>

std::map<int, Interpolator*> interpolatorObjects;

int Interpolator_create(){
    int maxObjects = 4096;
	int id = 0;
	while (interpolatorObjects.find(id) != interpolatorObjects.end())
	{
		id++;
		if (id > maxObjects)
			return -1;
	}
	//first check if the id exists.  if so, delete it and create a new one.
	interpolatorObjects[id] = new Interpolator();
	return id;
}

int Interpolator_open(int id, const char * filename){
	//TODO: error checking
	int status;
	if (interpolatorObjects.find(id) != interpolatorObjects.end()){
		//first, open the file
                std::string fName(filename);
		Interpolator * interpolator = interpolatorObjects[id];
//		std::cout << "before calling interpolator->open" << std::endl;
		status = interpolator->open(fName);
//		std::cout << "filename: " << filename << " status: " << status << " after" << std::endl;
		//kameleonObjects.erase(iter);
	} else {

		//should never happen
		std::cerr << "this should never happen!" << std::endl;
                return -1;
	}
//	std::cout << "new size of interpolatorObjects: " << interpolatorObjects.size() << std::endl;
	return status;
}

int Interpolator_load_variable(int id, const char * variable){
    int status = 0;
	if (interpolatorObjects.find(id) != interpolatorObjects.end()){
		
                std::string var(variable);
		Interpolator * interpolator = interpolatorObjects[id];
//		std::cout << "before calling interpolator->loadVariable" << std::endl;
		interpolator->loadVariable(var);
	} else {

		//should never happen
		std::cerr << "this should never happen!" << std::endl;
                return -1;
	}
//	std::cout << "new size of kameleonObjects: " << ccmc::kameleonObjects.size() << std::endl;
	return status;
}

float Interpolator_interpolate(int id, const char * variable, float * lat,
		float * lon){
    Interpolator * interpolator = interpolatorObjects[id];
    //std::string var(variable);
    //string* var1 = var;
    float value = interpolator->interpolate(variable, lat, lon);
    //std::cout<<"Inside Interpolator_interpolate: value = "<<value<<std::endl;
    return value;
     
    
}

void Interpolator_close(int id){
    //std::cout<<"Interpolator_close: before close call"<<std::endl;
    Interpolator* interpolator = interpolatorObjects[id];
    interpolator->close();
    //std::cout<<"Interpolator_close: after close call"<<std::endl;
}

int Interpolator_delete(int id){
    if (interpolatorObjects.find(id) != interpolatorObjects.end()){
        Interpolator* interpolator = interpolatorObjects[id];
        delete interpolator;
        interpolatorObjects.erase(id);
        return 0;
    } else {
        //should never happen
	std::cerr << "this should never happen!" << std::endl;
        return -1;
    }
    
}


