/* 
 * File:   TimeInterpolator.cpp
 * Author: mswindell
 * 
 * Created on December 5, 2013, 3:52 PM
 */

#include <algorithm>
#include "TimeInterpolator.h"
#include "Interpolator.h"
#include "string.h"
#include "Time.h"
#include "Utils.h"

TimeInterpolator::TimeInterpolator() {
    isSorted = false;
}

TimeInterpolator::~TimeInterpolator() {
}

void TimeInterpolator::addTimestep(const std::string& filename) {
    FileReader* fileReader = new FileReader();
    int status = fileReader->open(filename);
    if (status == FileReader::OK) {
        Time time = fileReader->getCurrentTime();
        this->timesteps.push_back(time);
        std::sort(timesteps.begin(), timesteps.end());
        current_files[time] = fileReader;
    }
    //std::cout << "array: " << std::endl;
    //for (int i = 0; i < timesteps.size(); i++) {
        //std::cout << "time[" << i << "]: " << timesteps[i] << std::endl;
    //}
}

Time TimeInterpolator::parseTime(const std::string& timeString) {
    //std::cout << "timeString inside parseTime: " << timeString << std::endl;
    //std::cout << "before declaring string" << std::endl;
    char timeString_c_str[EPOCH3_STRING_LEN + 1];
    //std::cout << "before string copy" << std::endl;
    strncpy(timeString_c_str, (char*) timeString.c_str(), EPOCH_STRING_LEN);
    //std::cout << "after string copy" << std::endl;
    timeString_c_str[4] = '-';
    timeString_c_str[7] = '-';
    //std::cout << "before creating epoch time with parseEPOCH3" << std::endl;
    double epoch = parseEPOCH3(timeString_c_str);
    //std::cout << "epoch value in parseTime: " << epoch << std::endl;
    long year, month, day, hour, minute, second, msec;
    year = month = day = hour = minute = second = msec = 1;
    EPOCHbreakdown(epoch, &year, &month, &day, &hour, &minute, &second, &msec);
    Time time;
    time.setYear((short) year);
    time.setMonth((short) month);
    time.setDay((short) day);
    time.setHour((short) hour);
    time.setMinute((short) minute);
    time.setSeconds((short) second);
    time.setMilliseconds((short) msec);
    return time;
}

std::string TimeInterpolator::toString(const Time& time) {
    char epochString[EPOCH3_STRING_LEN + 1];
    //std::cout << "time: " << time.toString() << time.getEpoch() << std::endl;
    encodeEPOCH3(time.getEpoch(), epochString);
    return epochString;
}

void TimeInterpolator::manageVariables(const std::vector<std::string>& variables, FileReader* fr) {
    const std::vector<std::string> loadedVariables = fr->getLoadedVariables();

    //unload any unnecessary variables
    for (int i = 0; i < loadedVariables.size(); i++) {
        if (std::find(variables.begin(), variables.end(), loadedVariables[i]) == variables.end()) {
            if (loadedVariables[i] != "phi" && loadedVariables[i] != "theta")
                fr->unloadVariable(loadedVariables[i]);
        }
    }

    for (int i = 0; i < variables.size(); i++) {
        //std::cout << "loading variable " << variables[i] << std::endl;
        if (std::find(loadedVariables.begin(), loadedVariables.end(), variables[i]) == loadedVariables.end())
            fr->loadVariable(variables[i]);
    }
}

void TimeInterpolator::manageVariables(const char * variables[], int numVars, FileReader* a) {
    std::vector<std::string> loadedVariables = a->getLoadedVariables();
    //unload any unnecessary variables
    for (int i = 0; i < loadedVariables.size(); i++) {
        int count = 0;
        for (int j = 0; j < numVars; j++) {
            if (loadedVariables[i] != variables[j])
                count++;

        }
        if (count == numVars && loadedVariables[i] != "phi" && loadedVariables[i] != "theta")
            a->unloadVariable(loadedVariables[i]);
    }
    for (int i = 0; i < numVars; i++) {
        if (std::find(loadedVariables.begin(), loadedVariables.end(), variables[i]) == loadedVariables.end()) {
            //std::cout << "loading variable " << variables[i] << " from inside manageVariables." << std::endl;
            a->loadVariable(variables[i]);
        }
    }
}

bool TimeInterpolator::manageMemory(const Time& time, const std::vector<std::string>& variables) {
    //std::cout << "time: " << time.toString() << ": " << time.getEpoch() << std::endl;
    //std::cout << "timesteps: " << TimeInterpolator::toString(timesteps[0]) << ": " << timesteps[0].getEpoch();
    //std::cout << " " << TimeInterpolator::toString(timesteps[1]) << ": " << timesteps[1].getEpoch() << std::endl;
    int index = Utils<Time>::binary_search(this->timesteps, 0, this->timesteps.size(), time);
    //std::cout << "manageMemory. index: " << index << std::endl;
    //found the timestep(s)
    if (index >-1) {

        //if it is an exact time match and the file is already one of the current_kameleon_files, load just one file
        if (this->timesteps[index] == time && this->current_files.find(this->timesteps[index]) != this->current_files.end()) {
            //just load in one file
            FileReader * found = this->current_files[time];
            manageVariables(variables, found);

        }//the timesteps are both within the list
        else if (index + 1 < this->timesteps.size() &&
                this->current_files.find(this->timesteps[index]) != this->current_files.end() &&
                this->current_files.find(this->timesteps[index + 1]) != this->current_files.end()) {



            //load the variables in the files
            FileReader * before = this->current_files[this->timesteps[index]];
            FileReader * after = this->current_files[this->timesteps[index + 1]];
            manageVariables(variables, before);
            manageVariables(variables, after);
        }
        return true;
    } else {
        //do nothing
        //std::cout << "index was not > -1. returning" << std::endl;
        return false;
    }

}

/**
 * Manages the memory and the opened files based on the time and variable specified.
 */
bool TimeInterpolator::manageMemory(const Time& time, const char * variables[], int numVars) {

    //std::cout << "time: " << time.toString() << " timesteps: " << TimeInterpolator::toString(timesteps[0]) << " " << TimeInterpolator::toString(timesteps[1]) << std::endl;
    int index = Utils<Time>::binary_search(this->timesteps, 0, this->timesteps.size(), time);

    //found the timestep(s)
    if (index >-1) {

        //if it is an exact time match and the file is already one of the current_kameleon_files, load just one file
        if (this->timesteps[index] == time && this->current_files.find(this->timesteps[index]) != this->current_files.end()) {
            //just load in one file
            FileReader * found = this->current_files[time];
            manageVariables(variables, numVars, found);

        }//the timesteps are both within the list
        else if (index + 1 < this->timesteps.size() &&
                this->current_files.find(this->timesteps[index]) != this->current_files.end() &&
                this->current_files.find(this->timesteps[index + 1]) != this->current_files.end()) {
            //load the variables in the files
            FileReader * before = this->current_files[this->timesteps[index]];
            FileReader * after = this->current_files[this->timesteps[index + 1]];

            manageVariables(variables, numVars, before);
            manageVariables(variables, numVars, after);
        }
        return true;
    } else {
        //do nothing
        //std::cout << "index was not > -1. returning" << std::endl;
        return false;
    }
}

void TimeInterpolator::closeAndCleanupMemory() {
    for (int i = 0; i < timesteps.size(); i++) {
        this->current_files[timesteps[i]]->close();
    }
    timesteps.clear();
    current_files.clear();
}

float TimeInterpolator::interpolate(const Time& time, const std::string& variable, const float& c0, const float& c1, const float& c2) {
    int index = Utils<Time>::binary_search(this->timesteps, 0, this->timesteps.size(), time);
    if (index > -1) {
        if (time == this->timesteps[index]) {
            Interpolator * a = new Interpolator((current_files[time]));
            float value = a->interpolate(variable, c0, c1, c2);
            delete a;
            a = NULL;
            return value;
        } else {
            Time atime = this->timesteps[index];
            Time btime = this->timesteps[index + 1];
            Interpolator * a = new Interpolator((current_files[atime]));
            Interpolator * b = new Interpolator((current_files[btime]));
            double duration = btime - atime;
            double deltatime = (time - atime);

            double r = deltatime / duration;
            float avalue = a->interpolate(variable, c0, c1, c2);
            float bvalue = b->interpolate(variable, c0, c1, c2);

            float deltaValue = bvalue - avalue;
            float value = avalue + deltaValue * r;
            //std::cout << "inside. interpolated value: " << value << std::endl;
            delete a;
            delete b;
            a = NULL;
            b = NULL;
            return value;
        }
    } else
        return
        -1.f;
}
